from seantis.dir.base.upgrades import reset_images, add_behavior_to_item
from seantis.dir.council.directory import ICouncilDirectory
from seantis.dir.council.item import ICouncilDirectoryItem


def upgrade_to_1000(context):

    reset_images(context, (ICouncilDirectory, ICouncilDirectoryItem))
    add_behavior_to_item(
        context, 'seantis.dir.council', ICouncilDirectoryItem
    )



import re

tag_strip_expression = re.compile(r'<[^<]+?>')


def unsafe_strip_tags(text):
    """ Unsafely removes the tags from a given text. Unsafe meaning that it
    can be tricked and should not be used for validating user input.

    It's mostly useful to find out if a text without tags acutally contains
    any content.

    """
    return re.sub(tag_strip_expression, '', text)



from itertools import groupby
from operator import attrgetter

from five import grok
from plone.namedfile.field import NamedImage
from collections import namedtuple, OrderedDict

from seantis.dir.base import directory
from seantis.dir.base import utils
from seantis.dir.base.interfaces import IDirectory
from seantis.dir.council import _


class ICouncilDirectory(IDirectory):
    """Extends the seantis.dir.base.directory.IDirectory"""

    image = NamedImage(
        title=_(u'Image'),
        required=False,
        default=None
    )


ICouncilDirectory.setTaggedValue('seantis.dir.base.omitted', [
    'cat1', 'cat2', 'cat3', 'cat4',
    'cat1_suggestions',
    'cat2_suggestions',
    'cat3_suggestions',
    'cat4_suggestions'
])


class CouncilDirectory(directory.Directory):

    def labels(self):
        return {
            'cat1': _(u'Town'),
            'cat2': _(u'Party'),
            'cat3': _(u'Committee'),
            'cat4': _(u'Function'),
        }

    def used_categories(self):
        return ('cat1', 'cat2', 'cat3', 'cat4')

    def unused_categories(self):
        return tuple()


DirectoryTag = namedtuple('DirectoryTag', ['name', 'url', 'category'])


class CouncilDirectoryView(directory.View, directory.DirectoryCatalogMixin):
    grok.name('view')
    grok.context(ICouncilDirectory)
    grok.require('zope2.View')

    itemsperpage = 250
    template = grok.PageTemplateFile('templates/directory.pt')

    def generate_tags(self, item):

        url = self.directory.absolute_url() + '?filter&%s=%s'

        for cat, label, value in item.categories:

            for value in utils.flatten(value):
                yield DirectoryTag(
                    url=url % (cat, value), name=value, category=cat
                )

    def tags(self, item):
        return list(self.generate_tags(item))

    def all_tags(self):

        labels = self.directory.labels()
        url = self.directory.absolute_url() + '?filter&%s=%s'

        result = OrderedDict()
        possible_values = self.catalog.grouped_possible_values()

        for cat, values in possible_values.items():
            values = sorted(values.keys())
            result[labels[cat]] = [(url % (cat, v), v) for v in values]

        return result

    def filter_tags(self):

        labels = self.directory.labels()
        descriptions = self.directory.descriptions()

        tags = []

        Tag = namedtuple('ItemTag', ['label', 'value', 'description'])

        for category in sorted(descriptions):

            if category not in self.used_terms:
                continue

            value = self.used_terms[category]

            if value in descriptions[category]:
                desc = descriptions[category][value]
            else:
                desc = None

            tags.append(Tag(labels[category], value, desc))

        result = {}

        for group, values in groupby(tags, attrgetter('label')):
            result[group] = list(values)

        return result


class ExtendedDirectoryViewlet(grok.Viewlet):
    grok.context(ICouncilDirectory)
    grok.name('seantis.dir.council.directory.detail')
    grok.require('zope2.View')
    grok.viewletmanager(directory.DirectoryViewletManager)

    template = grok.PageTemplateFile('templates/directorydetail.pt')



from five import grok
from zope.interface import Interface
from seantis.dir.base import core


class View(core.View):
    """A numer of macros for use with seantis.dir.base"""

    grok.context(Interface)
    grok.require('zope2.View')
    grok.name('seantis-dir-council-macros')

    template = grok.PageTemplateFile('templates/macros.pt')



from setuptools import setup, find_packages
import os

version = '1.5.1'

zug_require = [
    'izug.basetheme',
    'ftw.contentmenu'
]

setup(name='seantis.dir.council',
      version=version,
      description="Directory of Council Members",
      long_description=('\n'.join((
          open("README.md").read(),
          open(os.path.join("docs", "HISTORY.txt")).read()
      ))),
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
          'Framework :: Plone',
          'Framework :: Plone :: 4.3',
          'Intended Audience :: Developers',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Programming Language :: Python',
      ],
      keywords='council directory seantis plone dexterity',
      author='Seantis GmbH',
      author_email='info@seantis.ch',
      url='https://github.com/seantis/seantis.dir.council',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['seantis', 'seantis.dir'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'Plone',
          'plone.app.dexterity',
          'collective.autopermission',
          'collective.testcaselayer',
          'collective.dexteritytextindexer',
          'seantis.dir.base>=1.5',
          'ftw.inflator',
          'izug.basetheme'
      ],
      extras_require=dict(zug=zug_require),
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



import imghdr

from operator import attrgetter
from collections import namedtuple
from itertools import groupby
from urllib import urlopen
from five import grok

from collective.dexteritytextindexer import searchable
from plone.app.z3cform.wysiwyg import WysiwygFieldWidget
from plone.directives import form
from plone.namedfile.field import NamedImage as NamedImageField
from plone.namedfile import NamedImage
from zope.interface import Invalid
from zope.schema import TextLine, Text

from seantis.dir.base import item
from seantis.dir.base import core
from seantis.dir.base import utils
from seantis.dir.base.schemafields import Email, AutoProtocolURI
from seantis.dir.base.interfaces import (
    IFieldMapExtender, IDirectoryItem, IDirectoryCategorized
)

from seantis.dir.council.utils import unsafe_strip_tags
from seantis.dir.council.directory import ICouncilDirectory
from seantis.dir.council import _


class ICouncilDirectoryItem(IDirectoryItem):
    """Extends the seantis.dir.IDirectoryItem."""

    image = NamedImageField(
        title=_(u'Image'),
        required=False,
        default=None
    )

    searchable('street')
    street = TextLine(
        title=_(u'Street'),
        required=False,
        default=u''
    )

    searchable('zipcode')
    zipcode = TextLine(
        title=_(u'Zipcode'),
        required=False,
        default=u''
    )

    searchable('city')
    city = TextLine(
        title=_(u'Town'),
        required=False,
        default=u''
    )

    searchable('phone')
    phone = TextLine(
        title=_(u'Phone'),
        required=False,
        default=u''
    )

    searchable('fax')
    fax = TextLine(
        title=_(u'Fax'),
        required=False,
        default=u''
    )

    searchable('url')
    url = AutoProtocolURI(
        title=_(u'Internet Address'),
        required=False,
        default=None
    )

    searchable('email')
    email = Email(
        title=_(u'Email'),
        required=False,
        default=u''
    )

    searchable('information')
    form.widget(information=WysiwygFieldWidget)
    information = Text(
        title=_(u'Information'),
        required=False,
        default=u''
    )


# Ensure that the uploaded image at least has an image header.
@form.validator(field=ICouncilDirectoryItem['image'])
def validate_image(value):
    if not value:
        return

    if not imghdr.what(value.filename, value.data):
        raise Invalid(_(u'Unknown image format'))


class CouncilDirectoryItem(item.DirectoryItem):

    def address_components(self, include_links=True):
        if self.street:
            yield self.street

        if self.zipcode or self.city:
            yield ', '.join((self.zipcode, self.city))

        if self.email and include_links:
            yield '<a href="mailto:%(m)s">%(m)s</a>' % {'m': self.email}

        if self.url and include_links:
            yield '<a href="%(u)s">%(u)s</a>' % {'u': self.url}

    @property
    def address_lines(self):
        return '\n'.join('<li>%s</li>' % c for c in self.address_components())

    def address_spans(self, include_links=True):
        return u'<div class="item-address">{}</div>'.format(
            u', '.join(
                u'<span>{}</span>'.format(c)
                for c in self.address_components(include_links=include_links)
            )
        )

    def available_information(self):
        if not self.information:
            return u''

        if not unsafe_strip_tags(self.information).strip():
            return u''

        return u'<div class="item-information">{}</div>'.format(
            self.information
        )


class View(core.View):
    """Default view of a seantis.dir.council item."""
    grok.context(ICouncilDirectoryItem)
    grok.require('zope2.View')

    template = grok.PageTemplateFile('templates/item.pt')

    def tags(self):
        directory = self.context.aq_inner.aq_parent

        labels = directory.labels()
        descriptions = directory.descriptions()

        tags = []

        Tag = namedtuple('ItemTag', ['label', 'value', 'description'])
        categorized = IDirectoryCategorized(self.context)

        for category in sorted(descriptions):
            values = getattr(categorized, category) or []

            for value in sorted(utils.flatten(values)):
                if value in descriptions[category]:
                    desc = descriptions[category][value]
                else:
                    desc = None

                tags.append(Tag(labels[category], value, desc))

        # why u no accept generators zpt???
        result = {}

        for group, values in groupby(tags, attrgetter('label')):
            result[group] = list(values)

        return result


class ExtendedDirectoryItemFieldMap(grok.Adapter):
    """Adapter extending the import/export fieldmap of
    seantis.dir.council.item.

    """
    grok.context(ICouncilDirectory)
    grok.provides(IFieldMapExtender)

    def __init__(self, context):
        self.context = context

    def extend_import(self, itemmap):
        itemmap.typename = 'seantis.dir.council.item'
        itemmap.interface = ICouncilDirectoryItem

        extended = [
            'street',
            'zipcode',
            'city',
            'phone',
            'fax'
        ]

        itemmap.add_fields(extended, len(itemmap))

        # this should be temporary affair
        def on_object_add(obj, record):
            url = record[7]

            if url.startswith('http://kantonsrat'):
                result = urlopen(url + '/@@images/image')

                if result.getcode() == 200:
                    obj.image = NamedImage(result.read(), filename=u'image')

        itemmap.on_object_add = on_object_add
